#-------------------------------------------------------------------------------
# Name:        search
#-------------------------------------------------------------------------------
#!/usr/bin/env python


#-------------------------------------------------------------------------------
# imports we need
import sys
import os
#-------------------------------------------------------------------------------



#-------------------------------------------------------------------------------
# add path to main script directory
curpath = os.path.dirname(os.path.realpath(__file__))
parpath = os.path.normpath(os.path.join(curpath, "../../.."))
sys.path.append(parpath)

# now import helper and ask helper to set up any special paths for us
from TWLCHelper import TWLCHelper
TWLCHelper.PrepareWlcEnvironment()
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# helper modules
from lxml.html import soupparser
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# import the parent classes
from TSearch import *
#-------------------------------------------------------------------------------









#---------------------------------------------------------------------------
class TSearch_AlternativeTo(TSearch):

    def __init__(self):
        # parent init
        super(TSearch_AlternativeTo, self).__init__()

    def __del__(self):
        pass

    def get_searchenginename(self):
        return "user.alternativeto"

    def ParseDerivedCommandlineArgs(self):
        # parse any derived object specific commandline options
        pass

    def BuildSearchUrlFromSearchString(self, search):
        # build params
        params = "q=" + self.helper.urlquoteargs(search)
        # form url
        url = "http://www.alternative.to/search?" + params 
        #print url
        #exit(1)
        # return it
        return url
        
    def ParseWebPageContentIntoResults(self, search, resultcontent, baseresult):
        # parse content of page request into results
        baseurl = "http://www.alternative.to"
        etree = soupparser.fromstring(resultcontent)
        xpathx = "//div[@class='info']"
        restree = etree.xpath(xpathx)
        #
        results = []
        resultindex = 0
        for elt in restree:
            #print ""
            #print "RESULT " + str(resultindex)
            #print ""
            # XXX: do the result = thing after verifying all pieces exist
            linktitle = ""
            url = ""
            excerpthtml = ""
            # now get properties
            # sample:
            # <div class="info">
                # <div class="title">
                    # <a href="/Redo_Backup_and_Recovery,29191229#nav-080576806" class="">Redo Backup and Recovery</a>
                # </div>
                # <div class="desc">
                     # Open-Source drive imaging solution coming as ...
                # </div>
                # <div class="stats">
                    # <span>Rank: <span class="bold">0</span></span>
                    # <span>Comments: <span class="bold">0</span></span>
                    # <span>Alternatives: <span class="bold">8</span></span>
                # </div>
                # <div class="actions">
                     # ...
                # </div>
            # </div>
            for subelt in elt:
                cl = subelt.get("class")
                if cl == "title":
                    if len(subelt) > 0:
                        a = subelt[0]
                        linktitle = self.helper.UnEscapeHtmlText(a.text)
                        if a.get("href"):
                            url = baseurl + a.get("href")
                        else:
                            break
                    else:
                        break
                elif cl == "desc":
                    excerpthtml = self.helper.CleanHtmlExcerptLine(subelt.text)
            if not(linktitle and url and excerpthtml):
                continue
            # start with base result
            result = dict(baseresult)
            result["linktitle"] = linktitle
            result["url"] = url
            result["excerpthtml"] = excerpthtml
            result["search"] = search
            # score with first result highest
            scoreval = 100 - resultindex
            result["score"] = str(scoreval)
            # add it
            results.append(result)
            # increment index
            resultindex = resultindex + 1
            #return results

        return results
#---------------------------------------------------------------------------






























#---------------------------------------------------------------------------
def main(comlineargs=None):
    # make object
    searcher = TSearch_AlternativeTo()

    # and ask it to run with commandline options
    return searcher.RunWithCommandlineOptions(comlineargs)
#---------------------------------------------------------------------------




#-------------------------------------------------------------------------------
if __name__ == '__main__':
    main()
#-------------------------------------------------------------------------------










