#-------------------------------------------------------------------------------
# Helper script used to run embedded wlc stuff
#-------------------------------------------------------------------------------
#!/usr/bin/env python


#---------------------------------------------------------------------------
import sys
import os
import site
#---------------------------------------------------------------------------



#---------------------------------------------------------------------------
# globals
Wlc_StdOutErrCatcher_stdout = None
Wlc_StdOutErrCatcher_stderr = None
#---------------------------------------------------------------------------






#---------------------------------------------------------------------------
# from http://effbot.org/pyfaq/how-do-i-catch-the-output-from-pyerr-print-or-anything-that-prints-to-stdout-stderr.htm
class Wlc_StdOutErrCatcher:
    def __init__(self,stream):
        self.data = ''
        self.oldstream = stream
    def write(self, stuff):
        self.data = self.data + stuff
    def clear(self):
        self.data = ''
    def close(self):
        pass
#---------------------------------------------------------------------------








#---------------------------------------------------------------------------
def WlcGlobal_FindImportPathFromFullDirectoryPath(fullfilepath):
    #given a full filepath to a .py file, calculate the relative path to importing it as a module
    # then we convert \ to .
    # ok so we find Tools\scripts\ and remove everything before that
    # then we remove the ending .py
    fullfilepath=fullfilepath.replace("\\","/")
    fullfilepath_lower = fullfilepath.lower()
    searchstr="tools/scripts/"
    spos = fullfilepath_lower.find(searchstr)
    #print "WlcGlobal_FindImportPathFromFullDirectoryPath for '"+fullfilepath+"' with spos = "+str(spos)
    if (spos==-1):
        return ""
    subpath=fullfilepath[spos+len(searchstr):]
    subpath=subpath.replace("/",".")
    subpathlen=len(subpath);
    #print "WlcGlobal_FindImportPathFromFullDirectoryPath for '"+fullfilepath+"' with subpathlen = "+str(subpathlen)+" and last3 are "+subpath[subpathlen-3:]
    if (subpathlen<3 or subpath[subpathlen-3:].lower()!='.py'):
        return ""
    subpath=subpath[0:len(subpath)-3]

    # ok now we're going to TEST it to make sure it has the required __init__.py files in the path to it
    WlcGlobal_VerifyImportIsReachableOrRaiseException(subpath)

    return subpath



def WlcGlobal_VerifyImportIsReachableOrRaiseException(importnamewithpath):
    # raise exception if the path to this import doesnt have __init__.py in it
    # split into list
    ipath = importnamewithpath.split(".")
    ilen = len(ipath)
    # ok all but last are directories
    curpath = os.path.dirname(os.path.realpath(__file__))
    for i in range(ilen-1):
        pathpart=ipath[i]
        curpath+="/"+pathpart
        #print "WlcGlobal_VerifyImportIsReachableOrRaiseException CHECKING "+curpath
        if (not (os.path.isfile(curpath+"/__init__.py") or (os.path.isfile(curpath+"/__init__.pyd")) )):
            raise Exception("ERROR: The requested import "+importnamewithpath+" does not have the required __init__.py files each each directory on the path.  You need to place these files there to let python traverse the directory tree when importing.  Specifically missing file: "+curpath+"/__init__.py")
    return True


def WlcGlobal_InvokeLocalPythonScriptViaMainImport(scriptfname,argline):
    #import the script and run its main func
    # ATTN: we need to import and  invoke dynamically value
    # ATTN: and then we need to redirect stdout so we can get it
    #print "Run import of main from "+cachescriptfname+" with args "+argline
    #exit(1)
    importedscript = __import__(scriptfname,fromlist=["main"])
    #print importedscript
    retv=importedscript.main(argline)
    return retv


def WlcGlobal_InvokeLocalPythonScriptViaMainImport_CaptureOutput(scriptfname,argline):
    #import the script and run its main func
    # ATTN: we need to import and  invoke dynamically value
    # ATTN: and then we need to redirect stdout so we can get it
    #print "Run import of main from "+cachescriptfname+" with args "+argline
    #exit(1)

    print "THIS DOES NOT WORK YET"
    exit(1)

    importedscript = __import__(scriptfname,fromlist=["main"])
    #print importedscript

    local_StdOutErrCatcher_stdout = Wlc_StdOutErrCatcher(sys.stdout)
    local_StdOutErrCatcher_stderr = Wlc_StdOutErrCatcher(sys.stderr)
    retv=importedscript.main(argline)
    sys.stdout = local_StdOutErrCatcher_stdout.oldstream
    sys.stderr = local_StdOutErrCatcher_stderr.oldstream
    outstr = local_StdOutErrCatcher_stdout.data
    errstr = local_StdOutErrCatcher_stderr.data
    if (outstr!="" and errstr!=""):
        outstr+="\n"
    outstr+=errstr
    #
    return (retv,outstr)



def WlcGlobal_StdioRedirection_Initialize():
    global Wlc_StdOutErrCatcher_stdout, Wlc_StdOutErrCatcher_stderr
    # redirect stdio to capture it
    Wlc_StdOutErrCatcher_stdout = Wlc_StdOutErrCatcher(sys.stdout)
    Wlc_StdOutErrCatcher_stderr = Wlc_StdOutErrCatcher(sys.stderr)
    sys.stdout = Wlc_StdOutErrCatcher_stdout
    sys.stderr = Wlc_StdOutErrCatcher_stderr


def WlcGlobal_StdioRedirection_DeInitialize():
    global Wlc_StdOutErrCatcher_stdout, Wlc_StdOutErrCatcher_stderr
    # restore stdio to orig
    sys.stdout = Wlc_StdOutErrCatcher_stdout.oldstream
    sys.stderr = Wlc_StdOutErrCatcher_stderr.oldstream
#---------------------------------------------------------------------------










































#---------------------------------------------------------------------------
# Callable from C Code
#---------------------------------------------------------------------------



#---------------------------------------------------------------------------
def WlcGlobalCi_RunSimScriptFunc(scriptpath,comlineargs):
    # return "" on no error, or errortext on error
    retvcode=1
    errorstr=""
    #
    scriptfname = WlcGlobal_FindImportPathFromFullDirectoryPath(scriptpath)
    #print "WlcGlobal_FindImportPathFromFullDirectoryPath:cachescriptfname = "+cachescriptfname
    #exit(1)
    #
    if (scriptfname==""):
        # error getting importpath
        errorstr = "WlcGlobalCi_RunSimScriptFunc: ERROR GETTING PATH TO IMPORT MODULE FROM FILE PATH: "+scriptfname
        retvcode = 1
    else:
        #print "DEBUG: calling WlcGlobal_InvokeLocalPythonScriptViaMainImport("+scriptfname+","+comlineargs+")"
        retvcode = WlcGlobal_InvokeLocalPythonScriptViaMainImport(scriptfname,comlineargs)
        if (retvcode!=0):
            # this is an error
            errorstr="WlcGlobal_InvokeLocalPythonScriptViaMainImport for "+scriptfname+" returned errorcode "+str(retvcode)
        #return "BACK FROM RunSimScriptFunc with script '"+scriptpath+"' and args='"+comlineargs+"'. with retcode="+str(retvcode)

    return errorstr




def WlcGlobalCi_Communicate(querymain,querysub):
    # return some info
    retstr = ""

    if (querymain=="GetInfo"):
        # Get some info useful for debugging
        retstr += "site.getsitepackages() = " + ("; ".join(site.getsitepackages()))+"\n"
        retstr += "site.getuserbase() = " + site.getuserbase()+"\n"
        retstr += "site.getusersitepackages() = " + site.getusersitepackages()+"\n"
        retstr += "sys.path = " + ("; ".join(sys.path))+"\n"
    elif (querymain=="AddStuffFromHomeDir"):
        # this is important -- our host will call this with python home directory (e.g. C:\Python) and we will ask site to add home\Lib\site-packages, which will cause .pth files
        #  in that directory to be processed, bringing in installed eggs that are not by default loaded with embedded python for some reason i don't undersand
        homedir=querysub
        site.addsitedir(querysub+"\\Lib\\site-packages")
        retstr="ok"
    elif (querymain=="AddStuffFromScriptBaseDir"):
        # this is important -- our host will call this with python home directory (e.g. C:\Python) and we will ask site to add home\Lib\site-packages, which will cause .pth files
        #  in that directory to be processed, bringing in installed eggs that are not by default loaded with embedded python for some reason i don't undersand
        homedir=querysub
        site.addsitedir(querysub+"\\lib\\site-packages")
        retstr="ok"
    elif (querymain=="Initialize"):
        # initialize the code to prepare for c invocations
        # one goal here might be to redirect stdio and capture it
        WlcGlobal_StdioRedirection_Initialize()
        retstr="ok"
    elif (querymain=="DeInitialize"):
        # initialize the code to prepare for c invocations
        # one goal here might be to redirect stdio and capture it
        WlcGlobal_StdioRedirection_DeInitialize()
        retstr="ok"
    elif (querymain=="GetClearStdOutErr"):
        # return and then clear stdout and stedd redirected strings
        global Wlc_StdOutErrCatcher_stdout, Wlc_StdOutErrCatcher_stderr
        retstr = "\n".join([Wlc_StdOutErrCatcher_stdout.data,Wlc_StdOutErrCatcher_stderr.data])
        Wlc_StdOutErrCatcher_stdout.clear()
        Wlc_StdOutErrCatcher_stderr.clear()
    else:
        # error
        retr="$ERROR$"
        raise Exception("Error in call to WlcGlobalCi_Communicate; unknown query: "+querymain+".")
    return retstr
#---------------------------------------------------------------------------
















#---------------------------------------------------------------------------
# this is at end because circular imports cause python to get sick and ill and insane
from TWLCHelper import TWLCHelper
#---------------------------------------------------------------------------





#---------------------------------------------------------------------------
def main(comlineargs=None):
    # this is not invoked as a standalone
    print "Main function of wlcembedder has nothing to do. Done"
    print WlcGlobalCi_Communicate("GetInfo","")
    pass

if __name__ == '__main__':
    main()
#---------------------------------------------------------------------------
